<?php

//LIST
function scripts_build_page() {
  $scripts = scripts_get_scripts();

  if (is_array($scripts) && count($scripts) > 0) {
    foreach ($scripts as $key => $script) {
      $row = array(
        array(
          'data' => $key,
        ),
        array(
          'data' => $script['cron'] ? 'True' : 'False',
        ),
        array(
          'data' => $script['init'] ? 'True' : 'False',
        ),
        array(
          'data' => l(t('Edit'), 'admin/config/scripts/edit/' . $key) . " | " . l($script['type'] == 'plugin' ? t('Reset') : t('Delete'), 'admin/config/scripts/delete/' . $key),
        ),
        array(
          'data' => drupal_get_form('scripts_run_form', $key),
          'colspan' => 5,
        ),
      );
      $rows[] = $row;
    }

    $variables = array(
      'header' => array(
        t('Script Name'),
        t('Run on cron'),
        t('Run on page init'),
        t('Operations'),
        t('Run'),
      ),
      'rows' => $rows,
    );
    $output = theme('table', $variables);
  }
  else {
    $output = t('No scripts configured.');
    $output .= ' ';
    $output .= l(t('Add script'), 'admin/config/scripts/add');
  }

  return $output;
}

//CREATION AND UPDATE
function scripts_build_form($form, & $form_state, $scriptid = '') {
  $scripts = scripts_get_scripts();

  if (isset ($scripts[$scriptid])) {
    $script = $scripts[$scriptid];
  }

  $form = array(
    '#id' => 'scripts-build-form',
    '#validate' => array(
      'scripts_build_validate',
    ),
    '#submit' => array(
      'scripts_build_submit',
    ),
  );

  $form['script_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Script Name'),
    '#description' => t('This is the unique name of the script. It must contain only alphanumeric characters and underscores'),
    '#required' => TRUE,
    '#default_value' => $scriptid,
  );

  if ($script['type'] == 'plugin') {
    $form['script_name']['#attributes'] = array(
      'readonly' => TRUE,
    );
  }

  $form['script_description'] = array(
    '#type' => 'textfield',
    '#title' => t('Script Description'),
    '#description' => t('This description will appear on the Scripts administrative UI to tell you what this script is about.'),
    '#required' => TRUE,
    '#default_value' => $script['description'],
  );

  $form['script_body'] = array(
    '#type' => 'textfield',
    '#size' => 120,
    '#title' => t('Script Code'),
    '#description' => t('Write the body of your script here.'),
    '#required' => TRUE,
    '#default_value' => $script['body'],
  );

  $form['script_init'] = array(
    '#title' => t('Run on init?'),
    '#type' => 'checkbox',
    '#default_value' => $script['init'],
  );

  $form['script_cron'] = array(
    '#title' => t('Run on cron?'),
    '#type' => 'checkbox',
    '#default_value' => $script['cron'],
  );

  $form['cron_time'] = array(
    '#type' => 'textfield',
    '#title' => t('Cron run time every?'),
    '#description' => t('Run this script every ?? mins'),
    '#default_value' => $script['cron_time'] / 60,
  );

  $form['script_type'] = array(
    '#type' => 'value',
    '#default_value' => $script['type'] ? $script['type'] : 'custom',
  );

  if (!ini_get('safe_mode')) {
    $form['script_exe_limit'] = array(
      '#type' => 'textfield',
      '#title' => t('Maximum execution time'),
      '#description' => t('Here you can set the maximum execution time for this script, default value = ') . ini_get('max_execution_time'),
      '#default_value' => $script['exe_time'],
    );
  }
  else {
    $form['script_exe'] = array(
      '#title' => t('Maximum execution time'),
      '#description' => t('Sorry you can\'t change this value due to some php.ini configuration the default value = ') . ini_get('max_execution_time'),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function scripts_build_validate($form, & $form_state) {
  if (arg(3) != 'edit') {
    if (!preg_match('/^[A-Za-z][A-Za-z0-9]*(?:_[A-Za-z0-9]+)*$/', $form_state['values']['script_name'])) {
      form_set_error('script_name', t('Name must contain only alphanumeric characters and underscores , must start with alphabetical'));
    }

    $scripts = array();
    $scripts = unserialize(variable_get('scripts', ''));
    if (is_array($scripts)) {
      foreach ($scripts as $script) {
        if ($script['name'] == $form_state['values']['script_name']) {
          form_set_error('script_name', t('Name must be unique'));
        }
      }
    }
  }
}

function scripts_build_submit($form, & $form_state) {
  $scripts = array();
  $scripts = unserialize(variable_get('scripts', ''));

  $form_state['redirect'] = 'admin/config/scripts';
  $script['name'] = $form_state['values']['script_name'];
  $script['description'] = $form_state['values']['script_description'];
  $script['body'] = $form_state['values']['script_body'];
  $script['init'] = $form_state['values']['script_init'];
  $script['cron'] = $form_state['values']['script_cron'];
  $script['exe_time'] = $form_state['values']['script_exe_limit'];
  $script['args'] = $form_state['values']['script_args'];
  $script['type'] = $form_state['values']['script_type'];
  $script['cron_time'] = $form_state['values']['cron_time'] * 60;
  $script['latest_run'] = time();
  $scripts[$script['name']] = $script;
  $serialized = serialize($scripts);
  variable_set("scripts", $serialized);
}

//DELETION
function scripts_delete_form($form, &$form_state, $scriptid) {
  $form = array(
    '#id' => 'scripts-build-delete-form',
    '#submit' => array(
      'scripts_delete_form_submit',
    )
  );

  return confirm_form($form, t('Are you sure you want to delete the script from the system?'), 'admin/config/scripts', '', t('Delete'), t('Cancel'));
}

function scripts_delete_form_submit($form, & $form_state) {
  $scriptid = arg(4);
  $scripts = unserialize(variable_get("scripts", ""));
  unset ($scripts[$scriptid]);
  $serialized = serialize($scripts);
  variable_set("scripts", $serialized);

  $form_state['redirect'] = 'admin/config/scripts';
}

//RUN
function scripts_run_script($scriptid) {
  $scripts = array();
  $scripts = unserialize(variable_get('scripts', ''));
  if (isset ($scripts[$scriptid])) {
    $script = $scripts[$scriptid];
  }

  $args = arg();
  unset ($args[0], $args[1], $args[2]);

  $run = scripts_run($script['body'], $script['exe_time']);
  return $run;
}
